/* Computer design #3
 * Michael Robinson
 * 8 November 2001
 */

`timescale 1ns/10ps

module comp3( global_clk, reset_, run_sw, write_sw );

// State definitions
parameter T1 = 0;
parameter T2 = 1;
parameter T3 = 2;

input global_clk;
input reset_;

input run_sw;
input write_sw;

reg [2:0] state;
reg [7:0] rom1 [8191:0];
reg [7:0] rom2 [255:0];
reg [3:0] ram [16:0];

wire [7:0] opcode;
wire [3:0] alu_a;

wire run, run_;
wire clk;

wire acc_set;
wire mem_set;

wire [3:0] a_add;
wire [3:0] alu_b;
wire [7:0] pc;
wire [2:0] cond_inst;
wire [3:0] acc;
wire [3:0] cond;
wire [3:0] result;
wire [3:0] f_res;
wire [3:0] f_cond;

initial begin
   $readmemh( "rom1.dat", rom1 );
   $readmemh( "rom2.dat", rom2 );
   $readmemh( "ram.dat", ram );
end

// Clock generation
assign clk = run & global_clk;

// Opcode decoding
assign a_add = opcode[7:4];
assign cond_inst = opcode[3:1];
assign acc_set = ~opcode[0] & state[T3];
assign mem_set = ~(opcode[0] & state[T3]) & write_sw;

// Run switch synchronization
u74ls74 run_buf( .d(run_sw),
                 .q(run),
                 .q_(run_),
                 .s_(1'b1), .r_(1'b1),
                 .clk(global_clk) );

// State ring counter
always @( posedge clk or negedge reset_ ) begin
   if( reset_ == 0 )
      state <= 3'b001;
   else
   begin
      state[0] <= state[2];
      state[1] <= state[0];
      state[2] <= state[1];
   end
end

// Program counter and next state ROM
u74ls273 prog_cnt( .d(rom1[{pc,cond}]),
                   .q(pc),
                   .mr_(reset_),
                   .clk(state[T1]) );

// Memory datapaths
assign opcode = rom2[pc];
assign alu_a = ram[a_add];

always @(negedge mem_set) begin
   ram[a_add] <= result;
end

// Accumulator
u74ls175 acc( .d(result), .q(alu_b), .mr_(reset_), .clk(acc_set));

// ALU Result
u74ls273 res( .d({f_res, f_cond}), .q({result,cond}), .mr_(reset_), 
              .clk(state[T2]));

// ALU
alu alu_blk( .a(alu_a), .b(alu_b), .s(cond_inst), .f(f_res), .cond(f_cond));

endmodule

